//---------------------------------------------------------------------------
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
// Resource
//---------------------------------------------------------------------------
var res = {
	"app.title": "Seven Open",
	"app.OpenTask.removeDuplicateCommandLines": true,
	"app.OpenTask.confirmOpenMany": true,
	"app.OpenTask.confirmOpenMany.many": 5,
	"app.OpenTask.confirmOpenMany.message": "The program opens %1 files.",
	"app.OpenTask.executionInterval": 0, // In milliseconds
	"app.OpenSpec.targetPath": "Target.lnk",
	"app.OpenCommander.fileNameExtensions.shortcutFile": ["lnk"],
	"app.OpenCommander.fileNameExtensions.archiveFile": [
		"001", "7z", "arj", "bz2", "bzip2", "cab", "cpio", "deb", "dmg",
		"fat", "gz", "gzip", "hfs", "iso", "lha", "lzh", "lzma", "ntfs",
		"rar", "rpm", "squashfs", "swm", "tar", "taz", "tbz", "tbz2", "tgz",
		"tpz", "txz", "vhd", "wim", "xar", "xz", "z", "zip"],
		// Supported on 7-Zip File Manager 9.20 (the latest stable) and 9.22
		// beta (the latest beta)
	"app.HelpTask.confirmOpenHelp": true,
	"app.HelpTask.confirmOpenHelp.message": "The program opens the offline help.",
	"app.HelpTask.helpPath": "Help\\index.html"
};
//---------------------------------------------------------------------------
// Library
//---------------------------------------------------------------------------
var lib = {};
lib.getHelpers = function() {
	var helpers = {};
	lib.getHelpers = function() { return helpers; }
	return lib.getHelpers.apply(this, arguments);
}
lib.getHelpers()["String.format"] = function(message) {
	var regExp = (new RegExp()).compile("%[\\d|%]", "g");
	var percentPattern = "%%";
	var percentLiteral = "%";
	lib.getHelpers()["String.format"] = function(message) {
		return message.replace(regExp, function(text) {
			if(text === percentPattern) {
				return percentLiteral;
			}
			return lib.getHelpers()["String.format"].arguments[parseInt(text.charAt(1), 10)]; }); }
	return lib.getHelpers()["String.format"].apply(this, arguments);
}
lib.getHelpers()["WshArguments.toArray"] = function(wshArguments) {
	var array = [];
	for(var i = 0; i < wshArguments.length; ++i) {
		array.push(wshArguments.item(i));
	}
	return array;
}
//---------------------------------------------------------------------------
// Application
//---------------------------------------------------------------------------
var app = {};
app.start = function() {
	try {
		if(WScript.arguments.length > 0) {
			(new app.OpenTask()).perform();
		}
		else {
			(new app.HelpTask()).perform();
		}
	}
	catch(exception) {
		if(exception === app.abort) {
			// Do nothing
		}
		else {
			throw exception;
		}
	}
}
app.resolvePath = function(path) {
	if(app.getFileSystem().getDriveName(path) > 0) {
		return path;
	}
	return app.getFileSystem().getAbsolutePathName(app.getFileSystem().buildPath(
		app.getFileSystem().getParentFolderName(WScript.scriptFullName), path));
}
app.abort = function() {
	throw app.abort;
}
app.getFileSystem = function() {
	var fileSystem = new ActiveXObject("Scripting.FileSystemObject");
	app.getFileSystem = function() { return fileSystem; }
	return app.getFileSystem.apply(this, arguments);
}
app.getShell = function() {
	var shell = new ActiveXObject("WScript.Shell");
	app.getShell = function() { return shell; }
	return app.getShell.apply(this, arguments);
}
app.getTitle = function() {
	return res["app.title"];
}
//---------------------------------------------------------------------------
app.Task = function() {
	// Do nothing
}
//---------------------------------------------------------------------------
app.OpenTask = function() {
	app.Task.apply(this, arguments);
}
app.OpenTask.prototype = new app.Task();
app.OpenTask.prototype.constructor = app.OpenTask;
app.OpenTask.prototype.perform = function() {
	var commandLines = this._createCommandLines();
	commandLines = this._ensureremoveDuplicateCommandLines(commandLines);
	commandLines = this._ensureConfirmOpenMany(commandLines);
	for(var i = 0; i < commandLines.length; ++i) {
		app.getShell().run(commandLines[i]);
		WScript.sleep(res["app.OpenTask.executionInterval"]);
	}
}
app.OpenTask.prototype._createCommandLines = function() {
	var commandLines = [];
	var commander = new app.OpenCommander(new app.OpenSpec(WScript.arguments));
	while(commander.hasNext()) {
		commandLines.push(commander.getNext());
	}
	return commandLines;
}
app.OpenTask.prototype._ensureremoveDuplicateCommandLines = function(commandLines) {
	if(!res["app.OpenTask.removeDuplicateCommandLines"]) {
		return commandLines;
	}
	var history = new ActiveXObject("Scripting.Dictionary");
	history.compareMode = 1;
	for(var i = 0; i < commandLines.length; ++i) {
		if(history.exists(commandLines[i])) {
			continue;
		}
		history.add(commandLines[i], null);
	}
	return (new VBArray(history.keys())).toArray();
}
app.OpenTask.prototype._ensureConfirmOpenMany = function(commandLines) {
	if(!res["app.OpenTask.confirmOpenMany"] || commandLines.length < res["app.OpenTask.confirmOpenMany.many"]) {
		return commandLines;
	}
	if(app.getShell().popup(
		lib.getHelpers()["String.format"](res["app.OpenTask.confirmOpenMany.message"], commandLines.length),
		0,
		app.getTitle(),
		1 + 48) != 1) {
		app.abort();
	}
	return commandLines;
}
//---------------------------------------------------------------------------
app.OpenSpec = function(commandLineArguments) {
	this._sourcePaths = (function() {
		var _sourcePaths = lib.getHelpers()["WshArguments.toArray"](commandLineArguments.unnamed);
		for(var i = 0; i < _sourcePaths.length; ++i) {
			_sourcePaths[i] = app.getFileSystem().getAbsolutePathName(_sourcePaths[i]);
		}
		return _sourcePaths; })();
	this._targetPath = app.resolvePath(res["app.OpenSpec.targetPath"]);
}
app.OpenSpec.prototype.getSourcePaths = function() {
	return this._sourcePaths;
}
app.OpenSpec.prototype.getTargetPath = function() {
	return this._targetPath;
}
//---------------------------------------------------------------------------
app.OpenCommander = function(spec) {
	this._spec = spec;
	this._done = 0;
	this._shortcutFileResolver = new app.FileResolver(
		res["app.OpenCommander.fileNameExtensions.shortcutFile"],
		function(path) { return app.getShell().CreateShortcut(path).targetPath; },
		null);
	this._archiveFileResolver = new app.FileResolver(
		res["app.OpenCommander.fileNameExtensions.archiveFile"],
		null,
		function(path) { return app.getFileSystem().getParentFolderName(path); });
}
app.OpenCommander.prototype.getNext = function() {
	return [
		'"' + this._spec.getTargetPath() + '"',
		'"' + this._archiveFileResolver.execute(
			this._shortcutFileResolver.execute(
			this._spec.getSourcePaths()[this._done++])) + '"'].join(" ");
}
app.OpenCommander.prototype.hasNext = function() {
	return this._done < this._spec.getSourcePaths().length;
}
//---------------------------------------------------------------------------
app.FileResolver = function(fileNameExtensions, onMatched, onNotMatched) {
	this._fileNameExtensions = (function() {
		var _fileNameExtensions = new ActiveXObject("Scripting.Dictionary")
		_fileNameExtensions.compareMode = 1;
		for(var i = 0; i < fileNameExtensions.length; ++i) {
			_fileNameExtensions.add(fileNameExtensions[i], null);
		}
		return _fileNameExtensions; })();
	this._onMatched = onMatched;
	this._onNotMatched = onNotMatched;
}
app.FileResolver.prototype.execute = function(path) {
	if(!app.getFileSystem().FileExists(path)) {
		return path;
	}
	return this._fileNameExtensions.exists(app.getFileSystem().getExtensionName(path)) ?
		this._doMatched(path) :
		this._doNotMatched(path);
}
app.FileResolver.prototype._doMatched = function(path) {
	return this._onMatched !== null ? this._onMatched(path) : path;
}
app.FileResolver.prototype._doNotMatched = function(path) {
	return this._onNotMatched !== null ? this._onNotMatched(path) : path;
}
//---------------------------------------------------------------------------
app.HelpTask = function() {
	app.Task.apply(this, arguments);
}
app.HelpTask.prototype = new app.Task();
app.HelpTask.prototype.constructor = app.HelpTask;
app.HelpTask.prototype.perform = function() {
	this._ensureConfirmOpenHelp();
	app.getShell().run('"' + app.resolvePath(res["app.HelpTask.helpPath"]) + '"');
}
app.HelpTask.prototype._ensureConfirmOpenHelp = function() {
	if(!res["app.HelpTask.confirmOpenHelp"]) {
		return;
	}
	if(app.getShell().popup(res["app.HelpTask.confirmOpenHelp.message"], 0, app.getTitle(), 1 + 48) != 1) {
		app.abort();
	}
}
//---------------------------------------------------------------------------
// Entry point
//---------------------------------------------------------------------------
app.start();
//---------------------------------------------------------------------------
